@@ -42,7 +42,7 @@ impl FromStr for Language {
             "C" => Ok(Language::C),
             "cython" => Ok(Language::Cython),
             "Cython" => Ok(Language::Cython),
-            _ => Err(format!("Unrecognized Language: '{}'.", s)),
+            _ => Err(format!("Unrecognized Language: '{s}'.")),
         }
     }
 }
@@ -103,7 +103,7 @@ impl FromStr for LineEndingStyle {
             "lf" => Ok(Self::LF),
             "crlf" => Ok(Self::CRLF),
             "cr" => Ok(Self::CR),
-            _ => Err(format!("Unrecognized line ending style: '{}'.", s)),
+            _ => Err(format!("Unrecognized line ending style: '{s}'.")),
         }
     }
 }
@@ -126,7 +126,7 @@ impl FromStr for Braces {
             "same_line" => Ok(Braces::SameLine),
             "NextLine" => Ok(Braces::NextLine),
             "next_line" => Ok(Braces::NextLine),
-            _ => Err(format!("Unrecognized Braces: '{}'.", s)),
+            _ => Err(format!("Unrecognized Braces: '{s}'.")),
         }
     }
 }
@@ -152,7 +152,7 @@ impl FromStr for Layout {
             "vertical" => Ok(Layout::Vertical),
             "Auto" => Ok(Layout::Auto),
             "auto" => Ok(Layout::Auto),
-            _ => Err(format!("Unrecognized Layout: '{}'.", s)),
+            _ => Err(format!("Unrecognized Layout: '{s}'.")),
         }
     }
 }
@@ -180,7 +180,7 @@ impl FromStr for DocumentationStyle {
             "c++" => Ok(DocumentationStyle::Cxx),
             "doxy" => Ok(DocumentationStyle::Doxy),
             "auto" => Ok(DocumentationStyle::Auto),
-            _ => Err(format!("Unrecognized documentation style: '{}'.", s)),
+            _ => Err(format!("Unrecognized documentation style: '{s}'.")),
         }
     }
 }
@@ -201,7 +201,7 @@ impl FromStr for DocumentationLength {
         match s.to_lowercase().as_ref() {
             "short" => Ok(DocumentationLength::Short),
             "full" => Ok(DocumentationLength::Full),
-            _ => Err(format!("Unrecognized documentation style: '{}'.", s)),
+            _ => Err(format!("Unrecognized documentation style: '{s}'.")),
         }
     }
 }
@@ -253,7 +253,7 @@ impl FromStr for Style {
             "tag" => Ok(Style::Tag),
             "Type" => Ok(Style::Type),
             "type" => Ok(Style::Type),
-            _ => Err(format!("Unrecognized Style: '{}'.", s)),
+            _ => Err(format!("Unrecognized Style: '{s}'.")),
         }
     }
 }
@@ -287,7 +287,7 @@ impl FromStr for ItemType {
             "typedefs" => Typedefs,
             "opaque" => OpaqueItems,
             "functions" => Functions,
-            _ => return Err(format!("Unrecognized Style: '{}'.", s)),
+            _ => return Err(format!("Unrecognized Style: '{s}'.")),
         })
     }
 }
@@ -309,7 +309,7 @@ impl FromStr for SortKey {
         Ok(match &*s.to_lowercase() {
             "name" => Name,
             "none" => None,
-            _ => return Err(format!("Unrecognized sort option: '{}'.", s)),
+            _ => return Err(format!("Unrecognized sort option: '{s}'.")),
         })
     }
 }
@@ -493,6 +493,9 @@ pub struct StructConfig {
     /// Whether associated constants should be in the body. Only applicable to
     /// non-transparent structs, and in C++-only.
     pub associated_constants_in_body: bool,
+    /// The rename rule to apply to the struct name used for prefixing associated
+    /// constants
+    pub rename_associated_constant: RenameRule,
     /// The way to annotate this struct as #[must_use].
     pub must_use: Option<String>,
     /// The way to annotation this function as #[deprecated] without notes
@@ -749,7 +752,7 @@ impl FromStr for Profile {
         match s {
             "debug" | "Debug" => Ok(Profile::Debug),
             "release" | "Release" => Ok(Profile::Release),
-            _ => Err(format!("Unrecognized Profile: '{}'.", s)),
+            _ => Err(format!("Unrecognized Profile: '{s}'.")),
         }
     }
 }
@@ -879,6 +882,8 @@ impl ParseConfig {
 pub struct PtrConfig {
     /// Optional attribute to apply to pointers that are required to not be null
     pub non_null_attribute: Option<String>,
+    /// Optional attribute to apply to pointers that may be null
+    pub nullable_attribute: Option<String>,
 }
 
 /// Settings specific to Cython bindings.
@@ -1113,7 +1118,7 @@ impl Config {
         })?;
 
         let mut config = toml::from_str::<Config>(&config_text)
-            .map_err(|e| format!("Couldn't parse config file: {}.", e))?;
+            .map_err(|e| format!("Couldn't parse config file: {e}."))?;
         config.config_path = Some(StdPathBuf::from(file_name.as_ref()));
         Ok(config)
     }
