@@ -5,9 +5,13 @@
 use std::cmp::Ordering;
 use std::collections::HashSet;
 
-use crate::bindgen::ir::{ItemContainer, Path};
+use crate::bindgen::{
+    ir::{ItemContainer, Path},
+    library::Library,
+};
 
 /// A dependency list is used for gathering what order to output the types.
+#[derive(Default)]
 pub struct Dependencies {
     pub order: Vec<ItemContainer>,
     pub items: HashSet<Path>,
@@ -21,21 +25,38 @@ impl Dependencies {
         }
     }
 
+    pub fn add(&mut self, library: &Library, path: &Path) {
+        let Some(items) = library.get_items(path) else {
+            warn!(
+                "Can't find {path}. This usually means that this type was incompatible or not found."
+            );
+            return;
+        };
+        if self.items.contains(path) {
+            return;
+        }
+        self.items.insert(path.clone());
+        for item in &items {
+            item.deref().add_dependencies(library, self);
+        }
+        for item in items {
+            self.order.push(item);
+        }
+    }
+
     pub fn sort(&mut self) {
         // Sort untagged enums and opaque structs into their own layers because they don't
         // depend on each other or anything else.
         let ordering = |a: &ItemContainer, b: &ItemContainer| match (a, b) {
-            (&ItemContainer::Enum(ref x), &ItemContainer::Enum(ref y))
+            (ItemContainer::Enum(x), ItemContainer::Enum(y))
                 if x.tag.is_none() && y.tag.is_none() =>
             {
-                x.path.cmp(&y.path)
+                Ordering::Equal
             }
-            (&ItemContainer::Enum(ref x), _) if x.tag.is_none() => Ordering::Less,
-            (_, &ItemContainer::Enum(ref x)) if x.tag.is_none() => Ordering::Greater,
+            (ItemContainer::Enum(x), _) if x.tag.is_none() => Ordering::Less,
+            (_, ItemContainer::Enum(x)) if x.tag.is_none() => Ordering::Greater,
 
-            (&ItemContainer::OpaqueItem(ref x), &ItemContainer::OpaqueItem(ref y)) => {
-                x.path.cmp(&y.path)
-            }
+            (ItemContainer::OpaqueItem(x), ItemContainer::OpaqueItem(y)) => x.path.cmp(&y.path),
             (&ItemContainer::OpaqueItem(_), _) => Ordering::Less,
             (_, &ItemContainer::OpaqueItem(_)) => Ordering::Greater,
 
