@@ -7,7 +7,7 @@ use std::collections::HashMap;
 use std::io::Write;
 
 use syn::ext::IdentExt;
-use syn::{self, UnOp};
+use syn::UnOp;
 
 use crate::bindgen::config::{Config, Language};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
@@ -16,8 +16,10 @@ use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
     Struct, ToCondition, Type,
 };
+use crate::bindgen::language_backend::LanguageBackend;
 use crate::bindgen::library::Library;
-use crate::bindgen::writer::{Source, SourceWriter};
+use crate::bindgen::rename::{IdentifierType, RenameRule};
+use crate::bindgen::writer::SourceWriter;
 use crate::bindgen::Bindings;
 
 fn member_to_ident(member: &syn::Member) -> String {
@@ -28,7 +30,7 @@ fn member_to_ident(member: &syn::Member) -> String {
 }
 
 // TODO: Maybe add support to more std associated constants.
-fn to_known_assoc_constant(associated_to: &Path, name: &str) -> Option<String> {
+pub(crate) fn to_known_assoc_constant(associated_to: &Path, name: &str) -> Option<String> {
     use crate::bindgen::ir::{IntKind, PrimitiveType};
 
     if name != "MAX" && name != "MIN" {
@@ -74,7 +76,13 @@ fn to_known_assoc_constant(associated_to: &Path, name: &str) -> Option<String> {
         },
         _ => return None,
     };
-    Some(format!("{}_{}", prefix, name))
+    Some(format!("{prefix}_{name}"))
+}
+
+#[derive(Debug, Clone)]
+pub struct LiteralStructField {
+    pub value: Literal,
+    pub cfg: Option<Cfg>,
 }
 
 #[derive(Debug, Clone)]
@@ -100,7 +108,7 @@ pub enum Literal {
     Struct {
         path: Path,
         export_name: String,
-        fields: HashMap<String, Literal>,
+        fields: HashMap<String, LiteralStructField>,
     },
     Cast {
         ty: Type,
@@ -109,7 +117,27 @@ pub enum Literal {
 }
 
 impl Literal {
-    fn replace_self_with(&mut self, self_ty: &Path) {
+    pub fn add_dependencies(&self, library: &Library, out: &mut Dependencies) {
+        self.visit(&mut |lit| {
+            match lit {
+                Literal::Struct {
+                    ref path,
+                    export_name: _,
+                    fields: _,
+                }
+                | Literal::Path {
+                    associated_to: Some((ref path, _)),
+                    name: _,
+                } => {
+                    out.add(library, path);
+                }
+                _ => {}
+            }
+            true
+        });
+    }
+
+    pub fn replace_self_with(&mut self, self_ty: &Path) {
         match *self {
             Literal::PostfixUnaryOp { ref mut value, .. } => {
                 value.replace_self_with(self_ty);
@@ -131,10 +159,10 @@ impl Literal {
                 ref mut fields,
             } => {
                 if path.replace_self_with(self_ty) {
-                    *export_name = self_ty.name().to_owned();
+                    self_ty.name().clone_into(export_name);
                 }
                 for ref mut expr in fields.values_mut() {
-                    expr.replace_self_with(self_ty);
+                    expr.value.replace_self_with(self_ty);
                 }
             }
             Literal::Cast {
@@ -150,7 +178,7 @@ impl Literal {
             } => {
                 if let Some((ref mut path, ref mut export_name)) = *associated_to {
                     if path.replace_self_with(self_ty) {
-                        *export_name = self_ty.name().to_owned();
+                        self_ty.name().clone_into(export_name);
                     }
                 }
             }
@@ -183,7 +211,7 @@ impl Literal {
         }
     }
 
-    fn can_be_constexpr(&self) -> bool {
+    pub(crate) fn can_be_constexpr(&self) -> bool {
         !self.has_pointer_casts()
     }
 
@@ -202,7 +230,7 @@ impl Literal {
             Literal::FieldAccess { ref base, .. } => base.visit(visitor),
             Literal::Struct { ref fields, .. } => {
                 for (_name, field) in fields.iter() {
-                    if !field.visit(visitor) {
+                    if !field.value.visit(visitor) {
                         return false;
                     }
                 }
@@ -249,7 +277,7 @@ impl Literal {
             } => {
                 config.export.rename(export_name);
                 for lit in fields.values_mut() {
-                    lit.rename_for_config(config);
+                    lit.value.rename_for_config(config);
                 }
             }
             Literal::FieldAccess { ref mut base, .. } => {
@@ -313,16 +341,21 @@ impl Literal {
                     syn::BinOp::Ne(..) => "!=",
                     syn::BinOp::Ge(..) => ">=",
                     syn::BinOp::Gt(..) => ">",
-                    syn::BinOp::AddEq(..) => "+=",
-                    syn::BinOp::SubEq(..) => "-=",
-                    syn::BinOp::MulEq(..) => "*=",
-                    syn::BinOp::DivEq(..) => "/=",
-                    syn::BinOp::RemEq(..) => "%=",
-                    syn::BinOp::BitXorEq(..) => "^=",
-                    syn::BinOp::BitAndEq(..) => "&=",
-                    syn::BinOp::BitOrEq(..) => "|=",
-                    syn::BinOp::ShlEq(..) => "<<=",
-                    syn::BinOp::ShrEq(..) => ">>=",
+                    syn::BinOp::AddAssign(..) => "+=",
+                    syn::BinOp::SubAssign(..) => "-=",
+                    syn::BinOp::MulAssign(..) => "*=",
+                    syn::BinOp::DivAssign(..) => "/=",
+                    syn::BinOp::RemAssign(..) => "%=",
+                    syn::BinOp::BitXorAssign(..) => "^=",
+                    syn::BinOp::BitAndAssign(..) => "&=",
+                    syn::BinOp::BitOrAssign(..) => "|=",
+                    syn::BinOp::ShlAssign(..) => "<<=",
+                    syn::BinOp::ShrAssign(..) => ">>=",
+                    currently_unknown => {
+                        return Err(format!(
+                            "unsupported binary operator: {currently_unknown:?}"
+                        ))
+                    }
                 };
                 Ok(Literal::BinOp {
                     left: Box::new(l),
@@ -337,7 +370,7 @@ impl Literal {
                     syn::Lit::Byte(ref value) => Ok(Literal::Expr(format!("{}", value.value()))),
                     syn::Lit::Char(ref value) => Ok(Literal::Expr(match value.value() as u32 {
                         0..=255 => format!("'{}'", value.value().escape_default()),
-                        other_code => format!(r"U'\U{:08X}'", other_code),
+                        other_code => format!(r"U'\U{other_code:08X}'"),
                     })),
                     syn::Lit::Int(ref value) => {
                         let suffix = match value.suffix() {
@@ -381,12 +414,13 @@ impl Literal {
                     } => name,
                     _ => return Err(format!("Unsupported call expression. {:?}", *expr)),
                 };
-                let mut fields = HashMap::<String, Literal>::default();
+                let mut fields = HashMap::<String, LiteralStructField>::default();
                 for (index, arg) in args.iter().enumerate() {
                     let ident =
                         member_to_ident(&syn::Member::Unnamed(syn::Index::from(index))).to_string();
                     let value = Literal::load(arg)?;
-                    fields.insert(ident, value);
+                    let field = LiteralStructField { value, cfg: None };
+                    fields.insert(ident, field);
                 }
                 Ok(Literal::Struct {
                     path: Path::new(struct_name.clone()),
@@ -401,11 +435,13 @@ impl Literal {
                 ..
             }) => {
                 let struct_name = path.segments[0].ident.unraw().to_string();
-                let mut field_map = HashMap::<String, Literal>::default();
+                let mut field_map = HashMap::<String, LiteralStructField>::default();
                 for field in fields {
                     let ident = member_to_ident(&field.member).to_string();
+                    let cfg = Cfg::load(&field.attrs);
                     let value = Literal::load(&field.expr)?;
-                    field_map.insert(ident, value);
+                    let field = LiteralStructField { value, cfg };
+                    field_map.insert(ident, field);
                 }
                 Ok(Literal::Struct {
                     path: Path::new(struct_name.clone()),
@@ -450,7 +486,7 @@ impl Literal {
                             name: path.segments[1].ident.to_string(),
                         }
                     }
-                    _ => return Err(format!("Unsupported path expression. {:?}", path)),
+                    _ => return Err(format!("Unsupported path expression. {path:?}")),
                 })
             }
 
@@ -472,107 +508,6 @@ impl Literal {
             _ => Err(format!("Unsupported expression. {:?}", *expr)),
         }
     }
-
-    pub(crate) fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        match self {
-            Literal::Expr(v) => match (&**v, config.language) {
-                ("true", Language::Cython) => write!(out, "True"),
-                ("false", Language::Cython) => write!(out, "False"),
-                (v, _) => write!(out, "{}", v),
-            },
-            Literal::Path {
-                ref associated_to,
-                ref name,
-            } => {
-                if let Some((ref path, ref export_name)) = associated_to {
-                    if let Some(known) = to_known_assoc_constant(path, name) {
-                        return write!(out, "{}", known);
-                    }
-                    let path_separator = match config.language {
-                        Language::Cython | Language::C => "_",
-                        Language::Cxx => {
-                            if config.structure.associated_constants_in_body {
-                                "::"
-                            } else {
-                                "_"
-                            }
-                        }
-                    };
-                    write!(out, "{}{}", export_name, path_separator)
-                }
-                write!(out, "{}", name)
-            }
-            Literal::FieldAccess {
-                ref base,
-                ref field,
-            } => {
-                write!(out, "(");
-                base.write(config, out);
-                write!(out, ").{}", field);
-            }
-            Literal::PostfixUnaryOp { op, ref value } => {
-                write!(out, "{}", op);
-                value.write(config, out);
-            }
-            Literal::BinOp {
-                ref left,
-                op,
-                ref right,
-            } => {
-                write!(out, "(");
-                left.write(config, out);
-                write!(out, " {} ", op);
-                right.write(config, out);
-                write!(out, ")");
-            }
-            Literal::Cast { ref ty, ref value } => {
-                out.write(if config.language == Language::Cython {
-                    "<"
-                } else {
-                    "("
-                });
-                ty.write(config, out);
-                out.write(if config.language == Language::Cython {
-                    ">"
-                } else {
-                    ")"
-                });
-                value.write(config, out);
-            }
-            Literal::Struct {
-                export_name,
-                fields,
-                path,
-            } => {
-                match config.language {
-                    Language::C => write!(out, "({})", export_name),
-                    Language::Cxx => write!(out, "{}", export_name),
-                    Language::Cython => write!(out, "<{}>", export_name),
-                }
-
-                write!(out, "{{ ");
-                let mut is_first_field = true;
-                // In C++, same order as defined is required.
-                let ordered_fields = out.bindings().struct_field_names(path);
-                for ordered_key in ordered_fields.iter() {
-                    if let Some(lit) = fields.get(ordered_key) {
-                        if !is_first_field {
-                            write!(out, ", ");
-                        } else {
-                            is_first_field = false;
-                        }
-                        match config.language {
-                            Language::Cxx => write!(out, "/* .{} = */ ", ordered_key),
-                            Language::C => write!(out, ".{} = ", ordered_key),
-                            Language::Cython => {}
-                        }
-                        lit.write(config, out);
-                    }
-                }
-                write!(out, " }}");
-            }
-        }
-    }
 }
 
 #[derive(Debug, Clone)]
@@ -631,7 +566,15 @@ impl Constant {
         documentation: Documentation,
         associated_to: Option<Path>,
     ) -> Self {
-        let export_name = path.name().to_owned();
+        let export_name = match associated_to.clone() {
+            Some(associated_to) => path
+                .name()
+                .strip_suffix(associated_to.name())
+                .unwrap()
+                .to_owned(),
+            None => path.name().to_owned(),
+        };
+
         Self {
             path,
             export_name,
@@ -656,6 +599,7 @@ impl Item for Constant {
 
     fn add_dependencies(&self, library: &Library, out: &mut Dependencies) {
         self.ty.add_dependencies(library, out);
+        self.value.add_dependencies(library, out);
     }
 
     fn export_name(&self) -> &str {
@@ -674,6 +618,10 @@ impl Item for Constant {
         &mut self.annotations
     }
 
+    fn documentation(&self) -> &Documentation {
+        &self.documentation
+    }
+
     fn container(&self) -> ItemContainer {
         ItemContainer::Constant(self.clone())
     }
@@ -689,12 +637,17 @@ impl Item for Constant {
     fn resolve_declaration_types(&mut self, resolver: &DeclarationTypeResolver) {
         self.ty.resolve_declaration_types(resolver);
     }
+
+    fn generic_params(&self) -> &GenericParams {
+        GenericParams::empty()
+    }
 }
 
 impl Constant {
-    pub fn write_declaration<F: Write>(
+    pub fn write_declaration<F: Write, LB: LanguageBackend>(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
         associated_to_struct: &Struct,
     ) {
@@ -704,18 +657,22 @@ impl Constant {
         debug_assert!(config.structure.associated_constants_in_body);
         debug_assert!(config.constant.allow_static_const);
 
+        let condition = self.cfg.to_condition(config);
+        condition.write_before(config, out);
         if let Type::Ptr { is_const: true, .. } = self.ty {
             out.write("static ");
         } else {
             out.write("static const ");
         }
-        self.ty.write(config, out);
-        write!(out, " {};", self.export_name())
+        language_backend.write_type(out, &self.ty);
+        write!(out, " {};", self.export_name());
+        condition.write_after(config, out);
     }
 
-    pub fn write<F: Write>(
+    pub fn write<F: Write, LB: LanguageBackend>(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
         associated_to_struct: Option<&Struct>,
     ) {
@@ -729,7 +686,7 @@ impl Constant {
             return;
         }
 
-        let associated_to_transparent = associated_to_struct.map_or(false, |s| s.is_transparent);
+        let associated_to_transparent = associated_to_struct.is_some_and(|s| s.is_transparent);
 
         let in_body = associated_to_struct.is_some()
             && config.language == Language::Cxx
@@ -750,7 +707,21 @@ impl Constant {
             Cow::Borrowed(self.export_name())
         } else {
             let associated_name = match associated_to_struct {
-                Some(s) => Cow::Borrowed(s.export_name()),
+                Some(s) => {
+                    let name = s.export_name();
+                    let rules = s
+                        .annotations
+                        .parse_atom::<RenameRule>("rename-associated-constant");
+                    let rules = rules
+                        .as_ref()
+                        .unwrap_or(&config.structure.rename_associated_constant);
+
+                    if let Some(r) = rules.not_none() {
+                        r.apply(name, IdentifierType::Type)
+                    } else {
+                        Cow::Borrowed(name)
+                    }
+                }
                 None => {
                     let mut name = self.associated_to.as_ref().unwrap().name().to_owned();
                     config.export.rename(&mut name);
@@ -761,16 +732,15 @@ impl Constant {
             Cow::Owned(format!("{}_{}", associated_name, self.export_name()))
         };
 
-        let value = match self.value {
-            Literal::Struct {
-                ref fields,
-                ref path,
-                ..
-            } if out.bindings().struct_is_transparent(path) => fields.iter().next().unwrap().1,
-            _ => &self.value,
-        };
+        let mut value = &self.value;
+        while let Literal::Struct { path, fields, .. } = value {
+            if !out.bindings().struct_is_transparent(path) {
+                break;
+            }
+            value = &fields.iter().next().unwrap().1.value
+        }
 
-        self.documentation.write(config, out);
+        language_backend.write_documentation(out, self.documentation());
 
         let allow_constexpr = config.constant.allow_constexpr && self.value.can_be_constexpr();
         match config.language {
@@ -789,22 +759,22 @@ impl Constant {
                     out.write("const ");
                 }
 
-                self.ty.write(config, out);
-                write!(out, " {} = ", name);
-                value.write(config, out);
+                language_backend.write_type(out, &self.ty);
+                write!(out, " {name} = ");
+                language_backend.write_literal(out, value);
                 write!(out, ";");
             }
             Language::Cxx | Language::C => {
-                write!(out, "#define {} ", name);
-                value.write(config, out);
+                write!(out, "#define {name} ");
+                language_backend.write_literal(out, value);
             }
             Language::Cython => {
                 out.write("const ");
-                self.ty.write(config, out);
+                language_backend.write_type(out, &self.ty);
                 // For extern Cython declarations the initializer is ignored,
                 // but still useful as documentation, so we write it as a comment.
-                write!(out, " {} # = ", name);
-                value.write(config, out);
+                write!(out, " {name} # = ");
+                language_backend.write_literal(out, value);
             }
         }
 
