@@ -10,16 +10,17 @@ use crate::bindgen::config::{Config, Language};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use crate::bindgen::dependencies::Dependencies;
 use crate::bindgen::ir::{
-    AnnotationSet, AnnotationValue, Cfg, ConditionWrite, Documentation, Field, GenericArgument,
-    GenericParams, GenericPath, Item, ItemContainer, Literal, Path, Repr, ReprStyle, Struct,
-    ToCondition, Type,
+    AnnotationSet, AnnotationValue, Cfg, ConditionWrite, DeprecatedNoteKind, Documentation, Field,
+    GenericArgument, GenericParams, GenericPath, Item, ItemContainer, Literal, Path, Repr,
+    ReprStyle, Struct, ToCondition, Type,
 };
+use crate::bindgen::language_backend::LanguageBackend;
 use crate::bindgen::library::Library;
 use crate::bindgen::mangle;
 use crate::bindgen::monomorph::Monomorphs;
 use crate::bindgen::rename::{IdentifierType, RenameRule};
 use crate::bindgen::reserved;
-use crate::bindgen::writer::{ListType, Source, SourceWriter};
+use crate::bindgen::writer::{ListType, SourceWriter};
 
 #[allow(clippy::large_enum_variant)]
 #[derive(Debug, Clone)]
@@ -46,7 +47,7 @@ impl VariantBody {
         Self::Empty(AnnotationSet::new())
     }
 
-    fn annotations(&self) -> &AnnotationSet {
+    pub fn annotations(&self) -> &AnnotationSet {
         match *self {
             Self::Empty(ref anno) => anno,
             Self::Body { ref body, .. } => &body.annotations,
@@ -104,7 +105,11 @@ impl EnumVariant {
         config: &Config,
     ) -> Result<Self, String> {
         let discriminant = match variant.discriminant {
-            Some((_, ref expr)) => Some(Literal::load(expr)?),
+            Some((_, ref expr)) => {
+                let mut discriminant = Literal::load(expr)?;
+                discriminant.replace_self_with(self_path);
+                Some(discriminant)
+            }
             None => None,
         };
 
@@ -118,7 +123,7 @@ impl EnumVariant {
 
             if inline_tag_field {
                 res.push(Field::from_name_and_type(
-                    inline_name.map_or_else(|| "tag".to_string(), |name| format!("{}_tag", name)),
+                    inline_name.map_or_else(|| "tag".to_string(), |name| format!("{name}_tag")),
                     Type::Path(GenericPath::new(Path::new("Tag"), vec![])),
                 ));
             }
@@ -151,9 +156,10 @@ impl EnumVariant {
             annotations.add_default("derive-ostream", AnnotationValue::Bool(b));
         }
 
-        let body_rule = enum_annotations
-            .parse_atom::<RenameRule>("rename-variant-name-fields")
-            .unwrap_or(config.enumeration.rename_variant_name_fields);
+        let body_rule = enum_annotations.parse_atom::<RenameRule>("rename-variant-name-fields");
+        let body_rule = body_rule
+            .as_ref()
+            .unwrap_or(&config.enumeration.rename_variant_name_fields);
 
         let body = match variant.fields {
             syn::Fields::Unit => VariantBody::Empty(annotations),
@@ -256,6 +262,9 @@ impl EnumVariant {
         if let VariantBody::Body { ref body, .. } = self.body {
             body.add_dependencies(library, out);
         }
+        if let Some(ref d) = self.discriminant {
+            d.add_dependencies(library, out);
+        }
     }
 
     fn resolve_declaration_types(&mut self, resolver: &DeclarationTypeResolver) {
@@ -292,31 +301,6 @@ impl EnumVariant {
     }
 }
 
-impl Source for EnumVariant {
-    fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        let condition = self.cfg.to_condition(config);
-        // Cython doesn't support conditional enum variants.
-        if config.language != Language::Cython {
-            condition.write_before(config, out);
-        }
-        self.documentation.write(config, out);
-        write!(out, "{}", self.export_name);
-        if let Some(discriminant) = &self.discriminant {
-            if config.language == Language::Cython {
-                // For extern Cython declarations the enumerator value is ignored,
-                // but still useful as documentation, so we write it as a comment.
-                out.write(" #")
-            }
-            out.write(" = ");
-            discriminant.write(config, out);
-        }
-        out.write(",");
-        if config.language != Language::Cython {
-            condition.write_after(config, out);
-        }
-    }
-}
-
 #[derive(Debug, Clone)]
 pub struct Enum {
     pub path: Path,
@@ -332,17 +316,17 @@ pub struct Enum {
 
 impl Enum {
     /// Name of the generated tag enum.
-    fn tag_name(&self) -> &str {
+    pub(crate) fn tag_name(&self) -> &str {
         self.tag.as_deref().unwrap_or_else(|| self.export_name())
     }
 
     /// Enum with data turns into a union of structs with each struct having its own tag field.
-    fn inline_tag_field(repr: &Repr) -> bool {
+    pub(crate) fn inline_tag_field(repr: &Repr) -> bool {
         repr.style != ReprStyle::C
     }
 
     pub fn add_monomorphs(&self, library: &Library, out: &mut Monomorphs) {
-        if self.generic_params.len() > 0 {
+        if self.is_generic() {
             return;
         }
 
@@ -491,6 +475,10 @@ impl Item for Enum {
         &mut self.annotations
     }
 
+    fn documentation(&self) -> &Documentation {
+        &self.documentation
+    }
+
     fn container(&self) -> ItemContainer {
         ItemContainer::Enum(self.clone())
     }
@@ -516,6 +504,10 @@ impl Item for Enum {
         }
     }
 
+    fn generic_params(&self) -> &GenericParams {
+        &self.generic_params
+    }
+
     fn rename_for_config(&mut self, config: &Config) {
         config.export.rename(&mut self.export_name);
 
@@ -550,8 +542,10 @@ impl Item for Enum {
             }
         }
 
-        if config.enumeration.prefix_with_name
-            || self.annotations.bool("prefix-with-name").unwrap_or(false)
+        if self
+            .annotations
+            .bool("prefix-with-name")
+            .unwrap_or(config.enumeration.prefix_with_name)
         {
             let separator = if config.export.mangle.remove_underscores {
                 ""
@@ -569,10 +563,10 @@ impl Item for Enum {
             }
         }
 
-        let rules = self
-            .annotations
-            .parse_atom::<RenameRule>("rename-all")
-            .unwrap_or(config.enumeration.rename_variants);
+        let rules = self.annotations.parse_atom::<RenameRule>("rename-all");
+        let rules = rules
+            .as_ref()
+            .unwrap_or(&config.enumeration.rename_variants);
 
         if let Some(r) = rules.not_none() {
             self.variants = self
@@ -654,112 +648,43 @@ impl Item for Enum {
     }
 }
 
-impl Source for Enum {
-    fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        let size = self.repr.ty.map(|ty| ty.to_primitive().to_repr_c(config));
-        let has_data = self.tag.is_some();
-        let inline_tag_field = Self::inline_tag_field(&self.repr);
-        let tag_name = self.tag_name();
-
-        let condition = self.cfg.to_condition(config);
-        condition.write_before(config, out);
-
-        self.documentation.write(config, out);
-        self.generic_params.write(config, out);
-
-        // If the enum has data, we need to emit a struct or union for the data
-        // and enum for the tag. C++ supports nested type definitions, so we open
-        // the struct or union here and define the tag enum inside it (*).
-        if has_data && config.language == Language::Cxx {
-            self.open_struct_or_union(config, out, inline_tag_field);
-        }
-
-        // Emit the tag enum and everything related to it.
-        self.write_tag_enum(config, out, size, has_data, tag_name);
-
-        // If the enum has data, we need to emit structs for the variants and gather them together.
-        if has_data {
-            self.write_variant_defs(config, out);
-            out.new_line();
-            out.new_line();
-
-            // Open the struct or union for the data (**), gathering all the variants with data
-            // together, unless it's C++, then we have already opened that struct/union at (*) and
-            // are currently inside it.
-            if config.language != Language::Cxx {
-                self.open_struct_or_union(config, out, inline_tag_field);
-            }
-
-            // Emit tag field that is separate from all variants.
-            self.write_tag_field(config, out, size, inline_tag_field, tag_name);
-            out.new_line();
-
-            // Open union of all variants with data, only in the non-inline tag scenario.
-            // Cython extern declarations don't manage layouts, layouts are defined entierly by the
-            // corresponding C code. So we can inline the unnamed union into the struct and get the
-            // same observable result. Moreother we have to do it because Cython doesn't support
-            // unnamed unions.
-            if !inline_tag_field && config.language != Language::Cython {
-                out.write("union");
-                out.open_brace();
-            }
-
-            // Emit fields for all variants with data.
-            self.write_variant_fields(config, out, inline_tag_field);
-
-            // Close union of all variants with data, only in the non-inline tag scenario.
-            // See the comment about Cython on `open_brace`.
-            if !inline_tag_field && config.language != Language::Cython {
-                out.close_brace(true);
-            }
-
-            // Emit convenience methods for the struct or enum for the data.
-            self.write_derived_functions_data(config, out, tag_name);
-
-            // Emit the post_body section, if relevant.
-            if let Some(body) = config.export.post_body(&self.path) {
-                out.new_line();
-                out.write_raw_block(body);
-            }
-
-            // Close the struct or union opened either at (*) or at (**).
-            if config.language == Language::C && config.style.generate_typedef() {
-                out.close_brace(false);
-                write!(out, " {};", self.export_name);
-            } else {
-                out.close_brace(true);
-            }
-        }
-
-        condition.write_after(config, out);
-    }
-}
-
 impl Enum {
     /// Emit the tag enum and convenience methods for it.
     /// For enums with data this is only a part of the output,
     /// but for enums without data it's the whole output (modulo doc comments etc.).
-    fn write_tag_enum<F: Write>(
+    pub(crate) fn write_tag_enum<
+        F: Write,
+        LB: LanguageBackend,
+        WV: Fn(&mut LB, &mut SourceWriter<F>, &EnumVariant),
+    >(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
         size: Option<&str>,
-        has_data: bool,
-        tag_name: &str,
+        write_variant: WV,
     ) {
+        let tag_name = self.tag_name();
         // Open the tag enum.
         match config.language {
             Language::C => {
                 if let Some(prim) = size {
                     // If we need to specify size, then we have no choice but to create a typedef,
                     // so `config.style` is not respected.
-                    write!(out, "enum {}", tag_name);
+                    write!(out, "enum");
+                    if let Some(note) = self
+                        .annotations
+                        .deprecated_note(config, DeprecatedNoteKind::Enum)
+                    {
+                        write!(out, " {note}");
+                    }
+                    write!(out, " {tag_name}");
 
                     if config.cpp_compatible_c() {
                         out.new_line();
                         out.write("#ifdef __cplusplus");
                         out.new_line();
-                        write!(out, "  : {}", prim);
+                        write!(out, "  : {prim}");
                         out.new_line();
                         out.write("#endif // __cplusplus");
                         out.new_line();
@@ -769,8 +694,14 @@ impl Enum {
                         out.write("typedef ");
                     }
                     out.write("enum");
+                    if let Some(note) = self
+                        .annotations
+                        .deprecated_note(config, DeprecatedNoteKind::Enum)
+                    {
+                        write!(out, " {note}");
+                    }
                     if config.style.generate_tag() {
-                        write!(out, " {}", tag_name);
+                        write!(out, " {tag_name}");
                     }
                 }
             }
@@ -783,13 +714,20 @@ impl Enum {
 
                 if self.annotations.must_use(config) {
                     if let Some(ref anno) = config.enumeration.must_use {
-                        write!(out, " {}", anno)
+                        write!(out, " {anno}")
                     }
                 }
 
-                write!(out, " {}", tag_name);
+                if let Some(note) = self
+                    .annotations
+                    .deprecated_note(config, DeprecatedNoteKind::Enum)
+                {
+                    write!(out, " {note}");
+                }
+
+                write!(out, " {tag_name}");
                 if let Some(prim) = size {
-                    write!(out, " : {}", prim);
+                    write!(out, " : {prim}");
                 }
             }
             Language::Cython => {
@@ -809,13 +747,13 @@ impl Enum {
             if i != 0 {
                 out.new_line()
             }
-            variant.write(config, out);
+            write_variant(language_backend, out, variant);
         }
 
         // Close the tag enum.
         if config.language == Language::C && size.is_none() && config.style.generate_typedef() {
             out.close_brace(false);
-            write!(out, " {};", tag_name);
+            write!(out, " {tag_name};");
         } else {
             out.close_brace(true);
         }
@@ -841,11 +779,11 @@ impl Enum {
         }
 
         // Emit convenience methods for the tag enum.
-        self.write_derived_functions_enum(config, out, has_data, tag_name);
+        self.write_derived_functions_enum(config, language_backend, out);
     }
 
     /// The code here mirrors the beginning of `Struct::write` and `Union::write`.
-    fn open_struct_or_union<F: Write>(
+    pub(crate) fn open_struct_or_union<F: Write>(
         &self,
         config: &Config,
         out: &mut SourceWriter<F>,
@@ -861,10 +799,17 @@ impl Enum {
 
         if self.annotations.must_use(config) {
             if let Some(ref anno) = config.structure.must_use {
-                write!(out, " {}", anno);
+                write!(out, " {anno}");
             }
         }
 
+        if let Some(note) = self
+            .annotations
+            .deprecated_note(config, DeprecatedNoteKind::Struct)
+        {
+            write!(out, " {note} ");
+        }
+
         if config.language != Language::C || config.style.generate_tag() {
             write!(out, " {}", self.export_name());
         }
@@ -879,7 +824,12 @@ impl Enum {
     }
 
     /// Emit struct definitions for variants having data.
-    fn write_variant_defs<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
+    pub(crate) fn write_variant_defs<F: Write, LB: LanguageBackend>(
+        &self,
+        config: &Config,
+        language_backend: &mut LB, // TODO probably need only one of Config/LanguageBackend
+        out: &mut SourceWriter<F>,
+    ) {
         for variant in &self.variants {
             if let VariantBody::Body {
                 ref body,
@@ -894,7 +844,7 @@ impl Enum {
                 if config.language != Language::Cython {
                     condition.write_before(config, out);
                 }
-                body.write(config, out);
+                language_backend.write_struct(out, body);
                 if config.language != Language::Cython {
                     condition.write_after(config, out);
                 }
@@ -906,7 +856,7 @@ impl Enum {
     /// For non-inline tag scenario this is *the* tag field, and it does not exist in the variants.
     /// For the inline tag scenario this is just a convenience and another way
     /// to refer to the same tag that exist in all the variants.
-    fn write_tag_field<F: Write>(
+    pub(crate) fn write_tag_field<F: Write>(
         &self,
         config: &Config,
         out: &mut SourceWriter<F>,
@@ -927,7 +877,7 @@ impl Enum {
             out.write("enum ");
         }
 
-        write!(out, "{} tag;", tag_name);
+        write!(out, "{tag_name} tag;");
 
         if wrap_tag {
             out.close_brace(true);
@@ -935,11 +885,17 @@ impl Enum {
     }
 
     /// Emit fields for all variants with data.
-    fn write_variant_fields<F: Write>(
+    pub(crate) fn write_variant_fields<
+        F: Write,
+        LB: LanguageBackend,
+        WF: Fn(&mut LB, &mut SourceWriter<F>, &Field),
+    >(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
         inline_tag_field: bool,
+        write_field: WF,
     ) {
         let mut first = true;
         for variant in &self.variants {
@@ -970,7 +926,12 @@ impl Enum {
                     }
                     let start_field =
                         usize::from(inline_tag_field && config.language == Language::Cython);
-                    out.write_vertical_source_list(&body.fields[start_field..], ListType::Cap(";"));
+                    out.write_vertical_source_list(
+                        language_backend,
+                        &body.fields[start_field..],
+                        ListType::Cap(";"),
+                        &write_field,
+                    );
                     if config.language != Language::Cython {
                         out.close_brace(true);
                     }
@@ -987,13 +948,14 @@ impl Enum {
     }
 
     // Emit convenience methods for enums themselves.
-    fn write_derived_functions_enum<F: Write>(
+    fn write_derived_functions_enum<F: Write, LB: LanguageBackend>(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
-        has_data: bool,
-        tag_name: &str,
     ) {
+        let has_data = self.tag.is_some();
+        let tag_name = self.tag_name();
         if config.language != Language::Cxx {
             return;
         }
@@ -1048,7 +1010,7 @@ impl Enum {
                 );
                 out.new_line();
             }
-            write!(out, "switch ({})", instance);
+            write!(out, "switch ({instance})");
             out.open_brace();
             let vec: Vec<_> = self
                 .variants
@@ -1060,11 +1022,16 @@ impl Enum {
                     )
                 })
                 .collect();
-            out.write_vertical_source_list(&vec[..], ListType::Join(""));
+            out.write_vertical_source_list(
+                language_backend,
+                &vec[..],
+                ListType::Join(""),
+                |_, out, s| write!(out, "{s}"),
+            );
             out.close_brace(false);
             out.new_line();
 
-            write!(out, "return {};", stream);
+            write!(out, "return {stream};");
             out.close_brace(false);
 
             if has_data {
@@ -1092,7 +1059,7 @@ impl Enum {
                 );
                 out.new_line();
 
-                write!(out, "switch ({}.tag)", instance);
+                write!(out, "switch ({instance}.tag)");
                 out.open_brace();
                 let vec: Vec<_> = self
                     .variants
@@ -1121,22 +1088,33 @@ impl Enum {
                         }
                     })
                     .collect();
-                out.write_vertical_source_list(&vec[..], ListType::Join(""));
+                out.write_vertical_source_list(
+                    language_backend,
+                    &vec[..],
+                    ListType::Join(""),
+                    |_, out, s| write!(out, "{s}"),
+                );
                 out.close_brace(false);
                 out.new_line();
 
-                write!(out, "return {};", stream);
+                write!(out, "return {stream};");
                 out.close_brace(false);
             }
         }
     }
 
     // Emit convenience methods for structs or unions produced for enums with data.
-    fn write_derived_functions_data<F: Write>(
+    pub(crate) fn write_derived_functions_data<
+        F: Write,
+        LB: LanguageBackend,
+        WF: Fn(&mut LB, &mut SourceWriter<F>, &Field),
+    >(
         &self,
         config: &Config,
+        language_backend: &mut LB,
         out: &mut SourceWriter<F>,
         tag_name: &str,
+        write_field: WF,
     ) {
         if config.language != Language::Cxx {
             return;
@@ -1175,7 +1153,7 @@ impl Enum {
                 write!(out, "static {} {}(", self.export_name, variant.export_name);
 
                 if let VariantBody::Body { ref body, .. } = variant.body {
-                    let skip_fields = if body.has_tag_field { 1 } else { 0 };
+                    let skip_fields = body.has_tag_field as usize;
                     let vec: Vec<_> = body
                         .fields
                         .iter()
@@ -1188,7 +1166,12 @@ impl Enum {
                             )
                         })
                         .collect();
-                    out.write_vertical_source_list(&vec[..], ListType::Join(","));
+                    out.write_vertical_source_list(
+                        language_backend,
+                        &vec[..],
+                        ListType::Join(","),
+                        &write_field,
+                    );
                 }
 
                 write!(out, ")");
@@ -1202,7 +1185,7 @@ impl Enum {
                     ..
                 } = variant.body
                 {
-                    let skip_fields = if body.has_tag_field { 1 } else { 0 };
+                    let skip_fields = body.has_tag_field as usize;
                     for field in body.fields.iter().skip(skip_fields) {
                         out.new_line();
                         match field.ty {
@@ -1212,13 +1195,13 @@ impl Enum {
                                 write!(out, "for (int i = 0; i < {}; i++)", length.as_str());
                                 out.open_brace();
                                 write!(out, "::new (&result.{}.{}[i]) (", variant_name, field.name);
-                                ty.write(config, out);
+                                language_backend.write_type(out, ty);
                                 write!(out, ")({}[i]);", arg_renamer(&field.name));
                                 out.close_brace(false);
                             }
                             ref ty => {
                                 write!(out, "::new (&result.{}.{}) (", variant_name, field.name);
-                                ty.write(config, out);
+                                language_backend.write_type(out, ty);
                                 write!(out, ")({});", arg_renamer(&field.name));
                             }
                         }
@@ -1257,7 +1240,7 @@ impl Enum {
                         VariantBody::Empty(..) => return,
                     };
 
-                    let skip_fields = if body.has_tag_field { 1 } else { 0 };
+                    let skip_fields = body.has_tag_field as usize;
                     let field_count = body.fields.len() - skip_fields;
                     if field_count == 0 {
                         return;
@@ -1280,7 +1263,7 @@ impl Enum {
                             is_ref: true,
                             is_nullable: false,
                         };
-                        return_type.write(config, out);
+                        language_backend.write_type(out, &return_type);
                     } else if const_casts {
                         write!(out, "const {}&", body.export_name());
                     } else {
@@ -1294,7 +1277,7 @@ impl Enum {
                     out.open_brace();
                     write!(out, "{}(Is{}());", assert_name, variant.export_name);
                     out.new_line();
-                    write!(out, "return {}", member_name);
+                    write!(out, "return {member_name}");
                     if inline_casts {
                         write!(out, "._0");
                     }
@@ -1337,7 +1320,7 @@ impl Enum {
                 self.export_name, other
             );
             out.open_brace();
-            write!(out, "if (tag != {}.tag)", other);
+            write!(out, "if (tag != {other}.tag)");
             out.open_brace();
             write!(out, "return false;");
             out.close_brace(false);
@@ -1388,7 +1371,7 @@ impl Enum {
                     self.export_name, other
                 );
                 out.open_brace();
-                write!(out, "return !(*this == {});", other);
+                write!(out, "return !(*this == {other});");
                 out.close_brace(false);
             }
         }
@@ -1462,7 +1445,7 @@ impl Enum {
                 self.export_name, self.export_name, other
             );
             out.new_line();
-            write!(out, " : tag({}.tag)", other);
+            write!(out, " : tag({other}.tag)");
             out.open_brace();
             write!(out, "switch (tag)");
             out.open_brace();
@@ -1508,7 +1491,7 @@ impl Enum {
                     self.export_name, self.export_name, other
                 );
                 out.open_brace();
-                write!(out, "if (this != &{})", other);
+                write!(out, "if (this != &{other})");
                 out.open_brace();
                 write!(out, "this->~{}();", self.export_name);
                 out.new_line();
