@@ -262,7 +262,7 @@ impl ConstExpr {
                     syn::Lit::Int(ref len) => len.base10_digits().to_string(),
                     syn::Lit::Byte(ref byte) => u8::to_string(&byte.value()),
                     syn::Lit::Char(ref ch) => u32::to_string(&ch.value().into()),
-                    _ => return Err(format!("can't handle const expression {:?}", lit)),
+                    _ => return Err(format!("can't handle const expression {lit:?}")),
                 };
                 Ok(ConstExpr::Value(val))
             }
@@ -270,7 +270,8 @@ impl ConstExpr {
                 let generic_path = GenericPath::load(&path.path)?;
                 Ok(ConstExpr::Name(generic_path.export_name().to_owned()))
             }
-            _ => Err(format!("can't handle const expression {:?}", expr)),
+            syn::Expr::Cast(ref cast) => Ok(ConstExpr::load(&cast.expr)?),
+            _ => Err(format!("can't handle const expression {expr:?}")),
         }
     }
 
@@ -458,7 +459,7 @@ impl Type {
             syn::Type::Verbatim(ref tokens) if tokens.to_string() == "..." => {
                 Type::Primitive(PrimitiveType::VaList)
             }
-            _ => return Err(format!("Unsupported type: {:?}", ty)),
+            _ => return Err(format!("Unsupported type: {ty:?}")),
         };
 
         Ok(Some(converted))
@@ -559,7 +560,7 @@ impl Type {
                 is_nullable: false,
                 is_ref: false,
             }),
-            "Cell" => Some(generic.into_owned()),
+            "SyncUnsafeCell" | "UnsafeCell" | "Cell" => Some(generic.into_owned()),
             "ManuallyDrop" | "MaybeUninit" | "Pin" if config.language != Language::Cxx => {
                 Some(generic.into_owned())
             }
@@ -700,24 +701,7 @@ impl Type {
                 }
                 let path = generic.path();
                 if !generic_params.iter().any(|param| param.name() == path) {
-                    if let Some(items) = library.get_items(path) {
-                        if !out.items.contains(path) {
-                            out.items.insert(path.clone());
-
-                            for item in &items {
-                                item.deref().add_dependencies(library, out);
-                            }
-                            for item in items {
-                                out.order.push(item);
-                            }
-                        }
-                    } else {
-                        warn!(
-                            "Can't find {}. This usually means that this type was incompatible or \
-                             not found.",
-                            path
-                        );
-                    }
+                    out.add(library, path);
                 }
             }
             Type::Primitive(_) => {}
@@ -836,9 +820,8 @@ impl Type {
                     *generic_path = GenericPath::new(mangled_path.clone(), vec![]);
                 } else {
                     warn!(
-                        "Cannot find a mangling for generic path {:?}. This usually means that a \
-                         type referenced by this generic was incompatible or not found.",
-                        generic_path
+                        "Cannot find a mangling for generic path {generic_path:?}. This usually means that a \
+                         type referenced by this generic was incompatible or not found."
                     );
                 }
             }
