@@ -15,10 +15,10 @@ use crate::bindgen::cargo::{Cargo, PackageRef};
 use crate::bindgen::config::{Config, ParseConfig};
 use crate::bindgen::error::Error;
 use crate::bindgen::ir::{
-    AnnotationSet, Cfg, Constant, Documentation, Enum, Function, GenericParam, GenericParams,
-    ItemMap, OpaqueItem, Path, Static, Struct, Type, Typedef, Union,
+    AnnotationSet, AnnotationValue, Cfg, Constant, Documentation, Enum, Function, GenericParam,
+    GenericParams, ItemMap, OpaqueItem, Path, Static, Struct, Type, Typedef, Union,
 };
-use crate::bindgen::utilities::{SynAbiHelpers, SynAttributeHelpers, SynItemFnHelpers};
+use crate::bindgen::utilities::{SynAbiHelpers, SynAttributeHelpers, SynItemHelpers};
 
 const STD_CRATES: &[&str] = &[
     "std",
@@ -57,6 +57,7 @@ pub fn parse_src(src_file: &FilePath, config: &Config) -> ParseResult {
     };
 
     context.parse_mod(&pkg_ref, src_file, 0)?;
+    context.out.source_files = context.cache_src.keys().map(|k| k.to_owned()).collect();
     Ok(context.out)
 }
 
@@ -79,6 +80,14 @@ pub(crate) fn parse_lib(lib: Cargo, config: &Config) -> ParseResult {
 
     let binding_crate = context.lib.as_ref().unwrap().binding_crate_ref();
     context.parse_crate(&binding_crate)?;
+    context.out.source_files = context.cache_src.keys().map(|k| k.to_owned()).collect();
+    context.out.package_version = context
+        .lib
+        .as_ref()
+        .unwrap()
+        .binding_crate_ref()
+        .version
+        .unwrap();
     Ok(context.out)
 }
 
@@ -97,7 +106,7 @@ struct Parser<'a> {
     out: Parse,
 }
 
-impl<'a> Parser<'a> {
+impl Parser<'_> {
     fn should_parse_dependency(&self, pkg_name: &str) -> bool {
         if self.parsed_crates.contains(pkg_name) {
             return false;
@@ -122,7 +131,7 @@ impl<'a> Parser<'a> {
         // If we have a whitelist, check it
         if let Some(ref include) = self.config.parse.include {
             if !include.iter().any(|name| name == pkg_name) {
-                debug!("Excluding crate {}", pkg_name);
+                debug!("Excluding crate {pkg_name}");
                 return false;
             }
         }
@@ -336,8 +345,11 @@ impl<'a> Parser<'a> {
                     // Last chance to find a module path
                     let mut path_attr_found = false;
                     for attr in &item.attrs {
-                        if let Ok(syn::Meta::NameValue(syn::MetaNameValue { path, lit, .. })) =
-                            attr.parse_meta()
+                        if let syn::Meta::NameValue(syn::MetaNameValue {
+                            path,
+                            value: syn::Expr::Lit(syn::ExprLit { lit, .. }),
+                            ..
+                        }) = &attr.meta
                         {
                             match lit {
                                 syn::Lit::Str(ref path_lit) if path.is_ident("path") => {
@@ -406,6 +418,8 @@ pub struct Parse {
     pub opaque_items: ItemMap<OpaqueItem>,
     pub typedefs: ItemMap<Typedef>,
     pub functions: Vec<Function>,
+    pub source_files: Vec<FilePathBuf>,
+    pub package_version: String,
 }
 
 impl Parse {
@@ -419,6 +433,8 @@ impl Parse {
             opaque_items: ItemMap::default(),
             typedefs: ItemMap::default(),
             functions: Vec::new(),
+            source_files: Vec::new(),
+            package_version: String::new(),
         }
     }
 
@@ -466,6 +482,8 @@ impl Parse {
         self.opaque_items.extend_with(&other.opaque_items);
         self.typedefs.extend_with(&other.typedefs);
         self.functions.extend_from_slice(&other.functions);
+        self.source_files.extend_from_slice(&other.source_files);
+        self.package_version.clone_from(&other.package_version);
     }
 
     fn load_syn_crate_mod<'a>(
@@ -526,7 +544,9 @@ impl Parse {
                     if let syn::Type::Path(ref path) = *item_impl.self_ty {
                         if let Some(type_name) = path.path.get_ident() {
                             for method in item_impl.items.iter().filter_map(|item| match item {
-                                syn::ImplItem::Method(method) => Some(method),
+                                syn::ImplItem::Fn(method) if !method.should_skip_parsing() => {
+                                    Some(method)
+                                }
                                 _ => None,
                             }) {
                                 self.load_syn_method(
@@ -542,7 +562,7 @@ impl Parse {
                     }
                 }
                 syn::Item::Macro(ref item) => {
-                    self.load_builtin_macro(config, crate_name, mod_cfg, item)
+                    self.load_builtin_macro(config, crate_name, mod_cfg, item);
                 }
                 syn::Item::Mod(ref item) => {
                     nested_modules.push(item);
@@ -565,7 +585,11 @@ impl Parse {
         item_impl: &syn::ItemImpl,
     ) {
         let associated_constants = item_impl.items.iter().filter_map(|item| match item {
-            syn::ImplItem::Const(ref associated_constant) => Some(associated_constant),
+            syn::ImplItem::Const(ref associated_constant)
+                if !associated_constant.should_skip_parsing() =>
+            {
+                Some(associated_constant)
+            }
             _ => None,
         });
         self.load_syn_assoc_consts(
@@ -585,11 +609,12 @@ impl Parse {
         mod_cfg: Option<&Cfg>,
         item: &syn::ItemForeignMod,
     ) {
-        if !item.abi.is_c() {
-            info!("Skip {} - (extern block must be extern C).", crate_name);
+        if !item.abi.is_c() && !item.abi.is_omitted() {
+            info!("Skip {crate_name} - (extern block must be extern C).");
             return;
         }
 
+        let mod_cfg = Cfg::append(mod_cfg, Cfg::load(&item.attrs));
         for foreign_item in &item.items {
             if let syn::ForeignItem::Fn(ref function) = *foreign_item {
                 if !config
@@ -603,7 +628,14 @@ impl Parse {
                     return;
                 }
                 let path = Path::new(function.sig.ident.unraw().to_string());
-                match Function::load(path, None, &function.sig, true, &function.attrs, mod_cfg) {
+                match Function::load(
+                    path,
+                    None,
+                    &function.sig,
+                    true,
+                    &function.attrs,
+                    mod_cfg.as_ref(),
+                ) {
                     Ok(func) => {
                         info!("Take {}::{}.", crate_name, &function.sig.ident);
 
@@ -628,7 +660,7 @@ impl Parse {
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
         self_type: &Path,
-        item: &syn::ImplItemMethod,
+        item: &syn::ImplItemFn,
     ) {
         self.load_fn_declaration(
             config,
@@ -638,7 +670,6 @@ impl Parse {
             item,
             Some(self_type),
             &item.sig,
-            &item.vis,
             &item.attrs,
         )
     }
@@ -660,7 +691,6 @@ impl Parse {
             item,
             None,
             &item.sig,
-            &item.vis,
             &item.attrs,
         );
     }
@@ -672,10 +702,9 @@ impl Parse {
         binding_crate_name: &str,
         crate_name: &str,
         mod_cfg: Option<&Cfg>,
-        named_symbol: &dyn SynItemFnHelpers,
+        named_symbol: &dyn SynItemHelpers,
         self_type: Option<&Path>,
         sig: &syn::Signature,
-        vis: &syn::Visibility,
         attrs: &[syn::Attribute],
     ) {
         if !config
@@ -702,53 +731,29 @@ impl Parse {
         let is_extern_c = sig.abi.is_omitted() || sig.abi.is_c();
         let exported_name = named_symbol.exported_name();
 
-        if let syn::Visibility::Public(_) = vis {
-            match (is_extern_c, exported_name) {
-                (true, Some(exported_name)) => {
-                    let path = Path::new(exported_name);
-                    match Function::load(path, self_type, sig, false, attrs, mod_cfg) {
-                        Ok(func) => {
-                            info!("Take {}.", loggable_item_name());
-                            self.functions.push(func);
-                        }
-                        Err(msg) => {
-                            error!("Cannot use fn {} ({}).", loggable_item_name(), msg);
-                        }
+        match (is_extern_c, exported_name) {
+            (true, Some(exported_name)) => {
+                let path = Path::new(exported_name);
+                match Function::load(path, self_type, sig, false, attrs, mod_cfg) {
+                    Ok(func) => {
+                        info!("Take {}.", loggable_item_name());
+                        self.functions.push(func);
+                    }
+                    Err(msg) => {
+                        error!("Cannot use fn {} ({}).", loggable_item_name(), msg);
                     }
                 }
-                (true, None) => {
-                    warn!(
-                        "Skipping {} - (not `no_mangle`, and has no `export_name` attribute)",
-                        loggable_item_name()
-                    );
-                }
-                (false, Some(_exported_name)) => {
-                    warn!("Skipping {} - (not `extern \"C\"`", loggable_item_name());
-                }
-                (false, None) => {}
             }
-        } else {
-            match (is_extern_c, exported_name) {
-                (true, Some(..)) => {
-                    warn!(
-                        "Skipping {} - (not `pub` but is `extern \"C\"` and `no_mangle`)",
-                        loggable_item_name()
-                    );
-                }
-                (true, None) => {
-                    warn!(
-                        "Skipping {} - (not `pub` but is `extern \"C\"`)",
-                        loggable_item_name()
-                    );
-                }
-                (false, Some(..)) => {
-                    warn!(
-                        "Skipping {} - (not `pub` but is `no_mangle`)",
-                        loggable_item_name()
-                    );
-                }
-                (false, None) => {}
+            (true, None) => {
+                warn!(
+                    "Skipping {} - (not `no_mangle`, and has no `export_name` attribute)",
+                    loggable_item_name()
+                );
             }
+            (false, Some(_exported_name)) => {
+                warn!("Skipping {} - (not `extern \"C\"`)", loggable_item_name());
+            }
+            (false, None) => {}
         }
     }
 
@@ -765,7 +770,7 @@ impl Parse {
         let ty = match Type::load(impl_ty) {
             Ok(ty) => ty,
             Err(e) => {
-                warn!("Skipping associated constants for {:?}: {:?}", impl_ty, e);
+                warn!("Skipping associated constants for {impl_ty:?}: {e:?}");
                 return;
             }
         };
@@ -778,10 +783,7 @@ impl Parse {
         let impl_path = match ty.get_root_path() {
             Some(p) => p,
             None => {
-                warn!(
-                    "Couldn't find path for {:?}, skipping associated constants",
-                    ty
-                );
+                warn!("Couldn't find path for {ty:?}, skipping associated constants");
                 return;
             }
         };
@@ -793,7 +795,7 @@ impl Parse {
                 return;
             }
 
-            let path = Path::new(item.ident.unraw().to_string());
+            let path = Path::new(item.ident.unraw().to_string() + impl_path.name());
             match Constant::load(
                 path,
                 mod_cfg,
@@ -858,7 +860,7 @@ impl Parse {
 
                 let full_name = constant.path.clone();
                 if !self.constants.try_insert(constant) {
-                    error!("Conflicting name for constant {}", full_name);
+                    error!("Conflicting name for constant {full_name}");
                 }
             }
             Err(msg) => {
@@ -887,27 +889,18 @@ impl Parse {
             return;
         }
 
-        if let syn::Visibility::Public(_) = item.vis {
-            if item.is_no_mangle() {
-                match Static::load(item, mod_cfg) {
-                    Ok(constant) => {
-                        info!("Take {}::{}.", crate_name, &item.ident);
-
-                        self.globals.try_insert(constant);
-                    }
-                    Err(msg) => {
-                        warn!("Skip {}::{} - ({})", crate_name, &item.ident, msg);
-                    }
+        if let Some(exported_name) = item.exported_name() {
+            let path = Path::new(exported_name);
+            match Static::load(path, item, mod_cfg) {
+                Ok(constant) => {
+                    info!("Take {}::{}.", crate_name, &item.ident);
+                    self.globals.try_insert(constant);
+                }
+                Err(msg) => {
+                    warn!("Skip {}::{} - ({})", crate_name, &item.ident, msg);
                 }
             }
-        }
-
-        // TODO
-        if let syn::Visibility::Public(_) = item.vis {
         } else {
-            warn!("Skip {}::{} - (not `pub`).", crate_name, &item.ident);
-        }
-        if !item.is_no_mangle() {
             warn!("Skip {}::{} - (not `no_mangle`).", crate_name, &item.ident);
         }
     }
@@ -1017,18 +1010,26 @@ impl Parse {
         }
 
         let bitflags = match bitflags::parse(item.mac.tokens.clone()) {
-            Ok(b) => b,
+            Ok(bf) => bf,
             Err(e) => {
-                warn!("Failed to parse bitflags invocation: {:?}", e);
+                warn!("Failed to parse bitflags invocation: {e:?}");
                 return;
             }
         };
 
         let (struct_, impl_) = bitflags.expand();
-        self.load_syn_struct(config, crate_name, mod_cfg, &struct_);
-        // We know that the expansion will only reference `struct_`, so it's
-        // fine to just do it here instead of deferring it like we do with the
-        // other calls to this function.
-        self.load_syn_assoc_consts_from_impl(crate_name, mod_cfg, &impl_);
+        if let Some(struct_) = struct_ {
+            self.load_syn_struct(config, crate_name, mod_cfg, &struct_);
+        }
+        if let syn::Type::Path(ref path) = *impl_.self_ty {
+            if let Some(type_name) = path.path.get_ident() {
+                self.structs
+                    .for_items_mut(&Path::new(type_name.unraw().to_string()), |item| {
+                        item.annotations
+                            .add_default("internal-derive-bitflags", AnnotationValue::Bool(true));
+                    });
+            }
+        }
+        self.load_syn_assoc_consts_from_impl(crate_name, mod_cfg, &impl_)
     }
 }
